# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import configparser
import subprocess
import systemimage.config as sys_info
import time

from autopilot import introspection
from contextlib import contextmanager
from ubuntu_system_tests.helpers import click


@contextmanager
def override_proxy_timeout(proxy, timeout_seconds):
    original_timeout = proxy._poll_time
    try:
        proxy._poll_time = timeout_seconds
        yield proxy
    finally:
        proxy._poll_time = original_timeout


def get_proxy_object_for_existing_process(
        process_name, base_custom_proxy_object):
    """Return the proxy object for a currently running process."""
    return introspection.get_proxy_object_for_existing_process(
        pid=_get_pid(process_name),
        emulator_base=base_custom_proxy_object
    )


# Pulled from the UX tests
def _get_pid(process_name):
    """Returns pid for currently running browser process."""
    for i in range(10):
        try:
            return int(
                subprocess.check_output(
                    ['pidof', process_name]).strip()
            )
        except subprocess.CalledProcessError:
            time.sleep(1)
    else:
        raise RuntimeError(
            'Could not find autopilot interface for {} after 10 '
            'seconds'.format(process_name)
        )


"""

The following methods are for performing platform feature detection.

These methods may in future be moved into the autopilot.platform module.
Whilst the API will remain the same, the implementation of the methods may
change depening on the specific requirements of the supported devices.

"""


def get_device_name():
    """Return the device's model name.'"""
    return sys_info.config.device


def get_channel_name():
    """Return the build channel name."""
    parser = configparser.ConfigParser()
    parser.read('/etc/system-image/channel.ini')
    return parser['service']['channel']


def is_agps_supported():
    """Return True if a agps is supported, False if it is not."""
    agps_supported = True
    try:
        subprocess.check_call(
            ['dpkg-query --status ubuntu-location-provider-here'],
            shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
    except subprocess.CalledProcessError:
        agps_supported = False
    return agps_supported


def is_camera_supported():
    """Return True if a camera is supported, False if it is not."""
    return click.is_package_installed('com.ubuntu.camera')


def is_wifi_supported():
    """Return True if wifi is supported, False if it is not."""
    wifi_supported = True
    try:
        subprocess.check_call(
            ['nmcli -t -f type device | grep wifi'],
            shell=True, stdout=subprocess.DEVNULL)
    except subprocess.CalledProcessError:
        wifi_supported = False
    return wifi_supported


def is_bluetooth_supported():
    """Return True if Bluetooth is supported, False if it is not."""
    try:
        output = subprocess.check_output(['hcitool', 'dev'])
    except FileNotFoundError:
        return False
    devices = output.decode().splitlines()
    # first line always displays 'Devices:', so size needs to be > 1
    return len(devices) > 1
