# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import autopilot
import logging
import ubuntuuitoolkit as uitk

from camera_app.emulators.main_window import MainWindow
from camera_app.emulators.panel import Panel

logger = logging.getLogger(__name__)


class MainWindow(MainWindow):
    """Class to extend camera applications MainWindow controls."""

    @autopilot.logging.log_action(logger.info)
    def take_photo(self):
        """Check camera is ready and take a photo, return photoroll hint."""
        self.ensure_camera_ready()
        self.app.pointing_device.click_object(
            self.get_exposure_button())
        return self._get_photoroll_hint()

    def ensure_camera_ready(self):
        """Ensure that the camera is ready and exposure button displayed."""
        self.get_exposure_button().enabled.wait_for(True)

    def open_bottom_edge(self):
        """Swipe up to display the bottom edge."""
        self.ensure_camera_ready()
        bottom_edge = self.get_bottom_edge()
        bottom_edge.open()
        self._wait_for_fastblur(True)
        return bottom_edge

    def _get_fast_blur(self):
        """Get the fastblur object."""
        return self.app.wait_select_single('FastBlur')

    def _wait_for_fastblur(self, expected):
        """Wait for fastblur visibility to be required state."""
        self._get_fast_blur().visible.wait_for(expected)

    def _get_photoroll_hint(self):
        """Return the photoroll hint."""
        self.ensure_camera_ready()
        try:
            photoroll = self.app.select_single(PhotoRollHint, visible=True)
        except autopilot.exceptions.StateNotFoundError:
            raise Exception('Photo roll not displayed after taking photo')
        return photoroll

    def get_flash_icon_name(self):
        """Get the icon name for the flash button."""
        return self.get_flash_button().get_properties()['iconName']

    def set_flash_option(self, value):
        """Set the flash option to specified value."""
        self._press_flash_option()
        self._press_option(value)

    @autopilot.logging.log_action(logger.info)
    def _press_flash_option(self):
        """Press the flash option button."""
        self.app.pointing_device.click_object(self.get_flash_button())

    @autopilot.logging.log_action(logger.info)
    def _press_option(self, value):
        """Press the specified option button."""
        self.app.pointing_device.click_object(
            self.get_option_value_button(value))


class Panel(Panel):
    """Class to over-ride default Panel helper."""

    @autopilot.logging.log_action(logger.info)
    def _drag_to_close(self):
        """
        This method is required due to bug pad.lv/1428090.
        This should be removed as soon as this bug is fixed.
        """
        x, y, _, _ = self.globalRect
        line_x = x + self.width * 0.25
        start_y = y
        stop_y = y + self.height - 1
        self.pointing_device.drag(line_x, start_y, line_x, stop_y)


class PhotoRollHint(uitk.UbuntuUIToolkitCustomProxyObjectBase):
    """Class to represent the photoroll hint shown after taking first photo."""

    @autopilot.logging.log_action(logger.info)
    def swipe_to_dismiss(self):
        """Swipe left to dismiss hint and open gallery view."""
        x, y, w, h = self.globalRect
        x_mid = x + w // 2
        y_mid = y + h // 2
        self.pointing_device.drag(x_mid, y_mid, x, y_mid)
        self.visible.wait_for(False)
        return self._get_gallery_view()

    def _get_gallery_view(self):
        """Get the visible gallery view."""
        return self.get_root_instance().select_single(
            GalleryView, visible=True)


class GalleryView(uitk.UbuntuUIToolkitCustomProxyObjectBase):
    """Class to represent the photoroll gallery view."""

    def get_photo(self, path):
        """Look for the photo with the specified file path."""
        url_path = 'image://photo/file://{}'.format(path)
        return self.wait_select_single(
            'QQuickItem', photoUrl=url_path, visible=True)

    def go_back(self):
        """Go back to the camera app view."""
        self.header.go_back()

    @property
    def header(self):
        """Get the header object."""
        return self.select_single(GalleryViewHeader, visible=True)


class GalleryViewHeader(uitk.UbuntuUIToolkitCustomProxyObjectBase):
    """Class to represent the photoroll gallery view toolbar header."""

    @autopilot.logging.log_action(logger.info)
    def go_back(self):
        """Press the back button."""
        self.pointing_device.click_object(self._get_back_button())

    def _get_back_button(self):
        """Get the back button from the header."""
        return self.select_single(
            'IconButton', objectName='backButton', visible=True)
