# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import filecmp
import hashlib
import logging
import random
import os
import shutil
import string

logger = logging.getLogger(__name__)


def delete_file(file_name):
    """Delete the file passed as parameter. In case the file does not
    exist, the deletion is skipped"""
    if os.path.isfile(file_name):
        os.unlink(file_name)


def clean_files(dir_path):
    """Delete all the files in the directory.
    Subdirs are not deleted"""
    for root, dirs, files in os.walk(dir_path):
        for file in files:
            os.unlink(os.path.join(root, file))


def files_in_dir(dir_path):
    """Retrieve all the files in the directory"""
    f = []
    for (root, dirs, files) in os.walk(dir_path):
        f.extend(files)
        break
    return f


def remove_dir(dir_path):
    """Remove the directory"""
    shutil.rmtree(dir_path, True)


def clean_dir(dir_path):
    """Delete all the content of the directory"""
    remove_dir(dir_path)
    os.makedirs(dir_path)


def calculate_file_sha1(file_path):
    """Calculates the sha1 digest for the file
    :param file_path: The path to the file to calculate the sha1 digest
    :return The sha1 digest for the file passed as parameter
    :raise EnvironmentError: If the file couldn't be opened
    """
    sha = hashlib.sha1()
    with open(file_path, 'rb') as f:
        while True:
            block = f.read(1024)
            if not block:
                break
            sha.update(block)
        return sha.hexdigest()


def get_random_string(length=10):
    """Get a string with random content"""
    return ''.join(random.choice(string.ascii_uppercase) for i in
                   range(length))


def compare_files(file_1, file_2):
    """Return True if files match exactly, False otherwise."""
    return filecmp.cmp(file_1, file_2)


def create_random_file(path, ext='', length=10):
    """ Create a file with random name and content in the specified dir
    :param path: The path to create the file
    :param ext: The extension for the file
    :param length: the length of the file content
    :return: The path of the created file
    """
    file_path = os.path.join(path, get_random_string() + ext)
    if os.path.isfile(file_path):
        raise RuntimeError('The file to be created already exists')

    with open(file_path, 'w') as f:
        f.write(get_random_string(length))

    return file_path
