
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from autopilot import introspection
import ubuntuuitoolkit
from unity8 import indicators


class ModemInfoItem(ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase):

    def is_offline(self):
        return self.statusText == 'Offline'

    def is_inserted(self):
        """Return False if SIM ModemInfoItem reads 'No SIM', True otherwise.

        The SIM list is never empty--a dummy ModemInfoItem with text
        'No SIM' appears if no SIMs are inserted.  This function may
        be used for counting of *actual* SIMs, e.g.

        """
        return self.statusText != 'No SIM'


class NetworkIndicator():

    def __init__(self, main_window):
        super().__init__()
        self._main_window = main_window

    def open(self):
        """Open the network indicator page.

        :return: The custom proxy object for the network indicator page.

        """
        return self._main_window.open_indicator_page(
            'indicator-network')

    def close(self):
        """Close the indicator page."""
        self._main_window.close_indicator_page()

    def ensure_closed(self):
        """Ensure the indicator page is closed."""
        indicator_page = self._main_window.select_single(
            objectName='indicator-network-page')
        if indicator_page.visible:
            self.close()


class NetworkIndicatorPage(indicators.IndicatorPage):

    """Autopilot helper for the network indicator page."""
    @classmethod
    def validate_dbus_object(cls, path, state):
        name = introspection.get_classname_from_path(path)
        return (name == b'IndicatorPage' and
                state['objectName'][1] == 'indicator-network-page')

    def is_flight_mode_enabled(self):
        """Return True if flight mode is on, False otherwise."""
        return self._get_flight_mode_switch().checked

    def is_flight_mode_switch_visible(self):
        """Return True if flight mode switch is visible, False otherwise."""
        return self._get_flight_mode_switch().visible

    def _get_flight_mode_switch(self):
        return self.select_single(
            'SwitchMenu',
            objectName='indicator.airplane.enabled')

    def turn_on_flight_mode(self):
        """Actuate the flight mode switch to turn on as necessary."""
        if not self.is_flight_mode_enabled():
            self.pointing_device.click_object(
                self._get_flight_mode_switch())

    def turn_off_flight_mode(self):
        """Actuate the flight mode switch to turn off as necessary."""
        if self.is_flight_mode_enabled():
            self.pointing_device.click_object(
                self._get_flight_mode_switch())

    def is_wifi_enabled(self):
        """Return True if WiFi is enabled, False otherwise."""
        return self._get_wifi_switch().checked

    def is_wifi_switch_visible(self):
        """Return True if WiFi switch is visible, False otherwise."""
        return self._get_wifi_switch().visible

    def _get_wifi_switch(self):
        """Return the WiFi switch."""
        return self.select_single(
            'SwitchMenu',
            objectName='indicator.wifi.enable')

    def turn_on_wifi(self):
        """Actuate the WiFi switch to turn on as necessary."""
        if not self.is_wifi_enabled():
            self.pointing_device.click_object(
                self._get_wifi_switch())

    def turn_off_wifi(self):
        """Actuate the WiFi switch to turn off as necessary."""
        if self.is_wifi_enabled():
            self.pointing_device.click_object(
                self._get_wifi_switch())

    def _get_cellular_settings_menu_item(self):
        """Return the menu item which opens cellular settings."""
        return self.select_single(
            'StandardMenu',
            objectName='indicator.cellular.settings')

    def is_cellular_settings_visible(self):
        return self._get_cellular_settings_menu_item().visible

    def _get_wifi_settings_menu_item(self):
        """Return the menu item which opens WiFi settings."""
        return self.select_single(
            'StandardMenu',
            objectName='indicator.wifi.settings')

    def is_wifi_settings_visible(self):
        return self._get_wifi_settings_menu_item().visible

    def get_list_of_sims(self):
        """Return a list of ModemInfoItems, or empty list if none found.

        Note that we don't count the 'No SIM' item which appears if no
        SIM is found.

        """
        return [sim for sim in self.select_many('ModemInfoItem')
                if sim.is_inserted()]

    def is_sim_list_offline(self):
        """Return True if all SIMs are offline, False otherwise."""
        return all([sim.is_offline() for sim in self.get_list_of_sims()])

    def get_list_of_wifi_access_points(self):
        """Return a list of WiFi access points, empty list if none found."""
        return self.select_many('AccessPointMenu')
