# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import dbus
import logging

logger = logging.getLogger(__name__)

SIM_MANAGER = 'org.ofono.SimManager'


def _get_sim_manager(sim_number):
    bus = dbus.SystemBus()
    modem = bus.get_object('org.ofono', '/ril_{}'.format(sim_number))
    return dbus.Interface(modem, SIM_MANAGER)


def is_pin_locked(sim_number):
    sim_manager = _get_sim_manager(sim_number)
    locked_pins = sim_manager.GetProperties()['LockedPins']
    return 'pin' in locked_pins


def unlock_sim(sim_number, pin):
    sim_manager = _get_sim_manager(sim_number)
    if is_pin_locked(sim_number):
        sim_manager.UnlockPin('pin', pin)


def lock_sim(sim_number, pin):
    sim_manager = _get_sim_manager(sim_number)
    if not is_pin_locked(sim_number):
        sim_manager.LockPin('pin', pin)


def _check_modem(ril_modem, properties, modem_number=None,
                 ignore_phonesim=True):
    """Decide if the specified modem should be checked.

    Returns True if this modem should be checked, False if not.

    :param ignore_phonesim: If true then phonesim modems will be ignored.
    :param properties: Properties from the modem
    :param ril_modem: Name of the modem
    :parm modem_number: Number of specific modem to check, or None to check all
    :returns: True if this modem should be checked, False if not.
    """
    PHONESIM_SERIAL = '000000000000000'
    if (str(ril_modem).endswith(str(modem_number)) or modem_number is None):
        if 'Serial' not in properties:
            # This is not a real modem, ignore it
            check_status = False
        elif ignore_phonesim and properties['Serial'] == PHONESIM_SERIAL:
            # This is a phonesim modem and should be ignored
            check_status = False
        else:
            # This modem should be checked
            check_status = True
    else:
        check_status = False
    return check_status


def _get_all_modems(bus):
    """
    Get the DBus interfaces representing each modem
    the device has.

    :returns: A list of modems
    """
    manager = dbus.Interface(
        bus.get_object('org.ofono', '/'), 'org.ofono.Manager')
    return manager.GetModems()


def _sim_present(bus, ril_modem, properties,
                 sim_number=None, ignore_phonesim=True):
    if _check_modem(ril_modem, properties, modem_number=sim_number,
                    ignore_phonesim=ignore_phonesim):
        modem = bus.get_object('org.ofono', ril_modem)
        sim_manager = dbus.Interface(modem, SIM_MANAGER)
        try:
            if sim_manager.GetProperties()['Present']:
                return True
        except dbus.exceptions.DBusException as e:
            logger.warning('exception: {0}detected on modem: {1}'.format(
                           e, ril_modem))
    return False


def is_sim_inserted(sim_number=None, ignore_phonesim=True):
    """Check a SIM is inserted.

    Checks if a SIM card is inserted in the specified
    slot on the device. If no slot is specified then
    all slots will be checked.

    :param ignore_phonesim: If true then phonesim modems will be ignored.
    :param sim_number: The slot index to check for SIM presence

    :returns: True if a SIM is inserted in the specified slot or
    if no slot is specified then if a SIM is inserted in
    any slot, otherwise False
    """
    bus = dbus.SystemBus()
    for ril_modem, properties in _get_all_modems(bus):
        if _sim_present(bus, ril_modem, properties,
                        sim_number, ignore_phonesim):
            return True
    return False


def number_of_sims(ignore_phonesim=True):
    """
    Provide the number of SIMs inserted in the device.

    :returns: The number of SIMs found
    """
    number_of_sims = 0
    bus = dbus.SystemBus()
    for ril_modem, properties in _get_all_modems(bus):
        if _sim_present(bus, ril_modem, properties,
                        ignore_phonesim=ignore_phonesim):
            number_of_sims += 1
    return number_of_sims
