# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu Sanity Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import functools
import logging
import requests

from requests.auth import HTTPBasicAuth

logger = logging.getLogger(__name__)

TWILIO_API_URL = "https://api.twilio.com/2010-04-01"


class Telephony:

    def __init__(self, credentials, backend='twilio'):
        self._backend = _get_backend(backend)(credentials)

    def send_sms(self, to, from_, message):
        return self._backend.send_sms(to, from_, message)

    def get_sent_sms(self, to, from_, date_sent, body):
        return self._backend.get_sent_sms(to, from_, date_sent, body)

    def send_mms(self, to, from_, message, media):
        return self._backend.send_mms(to, from_, message, media)

    def make_phonecall(self, to, from_, hangup_timeout, voice_message):
        return self._backend.make_phonecall(to, from_, hangup_timeout,
                                            voice_message)


NeedBackendError = functools.partial(
    NotImplementedError,
    "This class cannot be instantiated. Please use a specific backend."
)


class TelephonyBackend:

    def __init__(self, credentials):
        """
        :param credentials: dictionary containing the required credential
          details specific for the backend.

        :raises ValueError: if any required credential details are missing.

        """
        raise NeedBackendError()

    def send_sms(self, to, from_, message):
        raise NeedBackendError()

    def get_sent_sms(self, to, from_, date_sent, body):
        raise NeedBackendError()

    def send_mms(self, to, from_, message, media):
        raise NeedBackendError()

    def make_phonecall(self, to, from_, hangup_timeout, voice_message):
        # Only a suggestion, not sure specifics what are required.
        raise NeedBackendError()


class TwilioBackend(TelephonyBackend):
    """Backend that utilises the Twilio service.

    Required credentials are:
      - AccountSid: This represents the account id
      - AuthToken: This is the token used to authenticate the request against
        the account (Note: It must be kept secret!).

    """

    def __init__(self, credentials):
        _raise_if_required_credentials_missing(
            credentials, 'AccountSid', 'AuthToken')
        self._account_sid = credentials['AccountSid']
        self._account_auth = credentials['AuthToken']
        self._http_auth = HTTPBasicAuth(self._account_sid, self._account_auth)

    def send_sms(self, to, from_, message):
        return self._send_message(to, from_, message)

    def send_mms(self, to, from_, message, media_url):
        return self._send_message(to, from_, message, media_url)

    def _send_message(self, to, from_, message, media_url=None):
        """
        Send an SMS or MMS message. If no media_url parameter is specified then
        the message will be sent as an SMS message. If a media_url parameter is
        specified then the content will be downloaded then attached to the
        message and sent as an MMS message.

        :param to: Phone number of recipient
        :param from_: Phone number of sender
        :param message: Body of the message to send
        :param media_url: Url of file to attach to message
        :return: True if message sent successfully, False otherwise
        """
        _raise_if_number_doesnt_look_valid(to)
        _raise_if_number_doesnt_look_valid(from_)
        url = self._get_api_url('Messages')
        params = {'From': from_, 'To': to, 'Body': message}
        if media_url:
            params['MediaUrl'] = media_url

        response = requests.post(url, data=params, auth=self._http_auth)
        if not response.ok:
            logger.error(response.text)
        return response.ok

    def get_sent_sms(self, to, from_, date_sent, body):
        _raise_if_number_doesnt_look_valid(to)
        _raise_if_number_doesnt_look_valid(from_)
        url = self._get_api_url('Messages')
        params = {'From': from_,
                  'To': to,
                  'DateSent': date_sent.strftime('%Y-%m-%d')}
        response = requests.get(url, data=params, auth=self._http_auth)
        if not response.ok:
            logger.error(response.text)
        for message in response.json()['messages']:
            if message['body'] == body:
                return message
        return None

    def make_phonecall(self, number, hangup_timeout, voice_message):
        _raise_if_number_doesnt_look_valid(number)
        url = self._get_api_url('Calls')  # NOQA

    def _get_api_url(self, action):
        """Return API URL to post to."""
        api_url = TWILIO_API_URL + '/Accounts/{sid}/{action}.json'.format(
            sid=self._account_sid,
            action=action
        )
        return api_url


class FakeBackend(TelephonyBackend):
    """A verified fake for use with testing.

    Does not make any calls to external resources nor does it intiate any
    actual calls/messages.

    """
    pass


def _make_rest_request(url, params, auth_details):
    """Return request object containing the results of the REST request."""
    # i.e. r = requests.post(....)
    # return r or should we check the status code here and raise an exception
    # if it's not what's expected (could be another arg.)
    pass


def _get_backend(backend):
    """Return the requested backend class

    :raises ValueError: if the requested backend was not found.

    """
    # i.e. return TwiloBackend (well, smarter than that)
    # The register (or whatever) could be a lot nicer too.

    backend_register = dict(
        twilio=TwilioBackend,
        fake=FakeBackend,
    )
    return backend_register[backend]


def _raise_if_number_doesnt_look_valid(number):
    """Ensure number looks valid (i.e. E.164 format).

    Does not ensure the number is in service etc.
    """
    pass


def _raise_if_required_credentials_missing(credentials, *required_keys):
    """Raise ValueError if any required keys are absent from credentials.

    The exception string will include a description of which keys are missing.

    """
    missing_keys = []
    for key in required_keys:
        if credentials.get(key, None) is None:
            missing_keys.append(key)

    if missing_keys:
        raise ValueError(
            "Missing required credential detail{plural}: '{details}'".format(
                plural='s' if len(missing_keys) > 1 else '',
                details=', '.join(missing_keys))
        )
