# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import fixtures
import os

from subprocess import check_call

NM_PATH = '/etc/NetworkManager/system-connections/'
ASKPASS_PATH = '/tmp/askpass.sh'


class AskPass(fixtures.Fixture):
    """Allows the user to call commands with sudo -A

    Creates an 'askpass' script which allows the user of the
    fixture to call commands with sudo -A when the fixture is
    loaded and cleans up the askpass script when finished
    """

    def __init__(self, device_password):
        super().__init__()
        self.device_password = device_password

    def setUp(self):
        super().setUp()
        self.useFixture(fixtures.EnvironmentVariable(
            'SUDO_ASKPASS', ASKPASS_PATH))
        with open(ASKPASS_PATH, 'w') as askpass:
            self.addCleanup(os.remove, ASKPASS_PATH)
            askpass.writelines('#!/bin/sh\necho {}'.format(
                self.device_password))
            os.fchmod(askpass.fileno(), 0o0744)


class RemoveNetworks(fixtures.Fixture):
    """Removes all networks from the device

    Removes all networks from the device and restarts
    network services to action that change and on cleanup
    restores the previous state
    """

    def __init__(self, device_password):
        super().__init__()
        self.device_password = device_password

    def setUp(self):
        super().setUp()
        # only backup networks if one already exists
        if self.is_network_present():
            self.tmpdir_fixture = fixtures.TempDir()
            self.useFixture(self.tmpdir_fixture)
            with AskPass(self.device_password):
                check_call('sudo -A mv {}/* {}/'.format(
                    NM_PATH, self.tmpdir_fixture.path), shell=True)
                # If we succesfully move the connection files
                # then we need to make sure to reinstate them
                self.addCleanup(self._restore_networks)
                check_call('sudo -A service network-manager restart',
                           shell=True)
            check_call(['initctl', 'restart', 'indicator-network'])

    def _restore_networks(self):
        with AskPass(self.device_password):
            check_call('sudo -A mv {}/* {}'.format(
                       self.tmpdir_fixture.path, NM_PATH), shell=True)
            check_call('sudo -A service network-manager restart', shell=True)
        check_call(['initctl', 'restart', 'indicator-network'])

    def is_network_present(self):
        """Return True if a network is currently present."""
        return len(os.listdir(NM_PATH)) > 0
