# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014, 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from retrying import retry

from autopilot import introspection
import ubuntuuitoolkit
from unity8 import process_helpers
from unity8.shell.emulators import dash as unity8_dash


def get_dash():
    """Return the Unity8 Dash autopilot custom proxy object."""
    pid = process_helpers.get_job_pid('unity8-dash')
    dash_proxy = introspection.get_proxy_object_for_existing_process(
        pid=pid,
        emulator_base=ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase,
    )
    return dash_proxy.select_single(Dash)


class Dash(unity8_dash.Dash):

    def wait_for_processing_to_complete(self):
        self.select_single(
            objectName='processingIndicator').visible.wait_for(False)

    def open_scope(self, scope):
        """Open the scope once the dash is ready to start the scroll
        :parameter scope: The name of the scope to open
        :return: The opened scope.
        """
        self.wait_for_dash_loaded()
        return super().open_scope(scope)

    def open_apps_scope(self):
        """Open the Apps scope once the dash is ready to start the scroll"""
        return self.open_scope('clickscope')

    def open_music_scope(self):
        """Open the Music scope once the dash is ready to start the scroll"""
        return self.open_scope('musicaggregator')

    def wait_for_dash_loaded(self):
        """Wait until the dash content is ready"""
        dash_content = self.select_single(objectName='dashContent')
        dash_content.pageHeaderTotallyVisible.wait_for(True)

    @retry(stop_max_attempt_number=3,
           wait_fixed=500,
           retry_on_exception=
           lambda exception: (isinstance(exception, AssertionError)))
    def _scroll_to_left_scope(self):
        """
        Call parent implementation, retrying in case it raises
        an exception due to a failed  scrolling
        """
        super()._scroll_to_left_scope()

    @retry(stop_max_attempt_number=3,
           wait_fixed=500,
           retry_on_exception=
           lambda exception: (isinstance(exception, AssertionError)))
    def _scroll_to_right_scope(self):
        """
        Call parent implementation, retrying in case it raises
        an exception due to a failed  scrolling
        """
        super()._scroll_to_right_scope()


class GenericScopeView(unity8_dash.GenericScopeView):

    def swipe_to_top(self):
        """Swipe to the top of the scope."""
        list_view = self.select_single(
            ListViewWithPageHeader, objectName='categoryListView')
        list_view.swipe_to_top()

    def click_scope_item(self, category, title, press_duration=0.10):
        """Override parent implementation making it click at the top of the
        application button to avoid issue when the button is not displayed
        completely

        :parameter category: The name of the category where the item is.
        :parameter title: The title of the item.

        """

        category_element = self._get_category_element(category)
        icon = category_element.wait_select_single('AbstractButton',
                                                   title=title)
        list_view = self.select_single(
            ListViewWithPageHeader, objectName='categoryListView')
        list_view.swipe_child_into_view(icon)

        x, y, width, height = icon.globalRect
        self.pointing_device.move(x + 1, y + 1)
        self.pointing_device.click(press_duration=press_duration)


class ListViewWithPageHeader(ubuntuuitoolkit.QQuickListView):

    def swipe_to_show_more_below(self, containers):
        # XXX overriden because the margin to start the swipe is bigger than
        # in the other apps. See bug http://pad.lv/1401517
        # --elopio - 2014-12-11
        if self.atYEnd:
            super().swipe_to_show_more_below(containers)
        else:
            start_x = stop_x = self.globalRect.x + (self.globalRect.width // 2)
            from ubuntuuitoolkit._custom_proxy_objects import _flickable
            start_y = _flickable._get_visible_container_bottom(containers) - 50
            stop_y = _flickable._get_visible_container_top(containers) + 5
            # The default drag rate was causing some applications to
            # be scrolled past. Slowing it down a few notches makes it
            # more reliable without being much slower.
            self._slow_drag(start_x, stop_x, start_y, stop_y, rate=4)
            self.dragging.wait_for(False)
            self.moving.wait_for(False)
