# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014, 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging
import os
import psutil
from retrying import retry
import subprocess
import time

from autopilot import (
    introspection,
    logging as autopilot_logging,
    testcase
)
import ubuntuuitoolkit
from ubuntuuitoolkit import fixture_setup
from unity8 import process_helpers

from ubuntu_system_tests import config
from ubuntu_system_tests.helpers import unity8
from ubuntu_system_tests.helpers.unity8 import dash
from ubuntu_system_tests.helpers.unity8.shell import Unity8  # NOQA

logger = logging.getLogger(__name__)


class BaseUbuntuSystemTestCase(testcase.AutopilotTestCase):

    @classmethod
    def setUpClass(cls):
        unity8.ensure_unity_stopped()
        super().setUpClass()

    @classmethod
    def tearDownClass(cls):
        super().tearDownClass()
        unity8.ensure_unity_running_and_greeter_hidden()

    @autopilot_logging.log_action(logger.info)
    def launch_unity(self):
        """Launch Unity8 with testability enabled.

        :return: The Unity8 autopiliot custom proxy object.

        """
        self.useFixture(
            fixture_setup.InitctlEnvironmentVariable(QT_LOAD_TESTABILITY='1'))
        process_helpers.start_job('unity8', 'QT_LOAD_TESTABILITY=1')
        unity = self.get_proxy_object('unity8')
        unity.wait_for_greeter_content()
        return unity

    # Retry added until fix for bug 1427946 in QtMir is landed
    @retry(stop_max_delay=30000,
           retry_on_exception=
           lambda exception: (isinstance(exception, psutil.NoSuchProcess) or
                              isinstance(exception, ValueError)))
    def get_proxy_object(self, pname):
        pid = process_helpers.get_job_pid(pname)
        unity = introspection.get_proxy_object_for_existing_process(
            pid=pid,
            emulator_base=ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase,
        )
        return unity

    def restart_unity(self):
        """Restart and unlock unity8"""
        unity8.ensure_unity_stopped()
        unity_proxy = self.launch_unity()
        unity_proxy.unlock()
        dash.get_dash().wait_for_dash_loaded()
        return unity_proxy

    def ensure_application_closed(self, app_name):
        """Stop the app before unity8 is restarted so it's not orphaned."""
        subprocess.call(['ubuntu-app-stop', app_name])

    def wait_until(self, predicate, timeout=10, period=1, *args, **kwargs):
        """Waits either until the timeout is reached or until the predicate
        evaluation is True.

        To evaluate the predicate
         :param: predicate: The method to be evaluated
         :param: timeout: The timeout for the polling
         :param: period: The retry period of time
         :param: *args: The predicate positional parameters
         :param: **kwargs: The predicate optional parameters
         :return: True if the predicate was evaluated as True before the
                  timeout is reached, False otherwise
        """
        end = time.time() + timeout
        while time.time() < end:
            if predicate(*args, **kwargs):
                return True
            time.sleep(period)
        return False

    def get_tests_dir(self):
        return os.path.dirname(os.path.realpath(__file__))


class BaseTelephonyTestCase(BaseUbuntuSystemTestCase):
    """Base class to be used for all telephony tests."""

    def setUp(self):
        """Base class setUp method."""
        super().setUp()
        self.config_stack = config.get_device_config_stack()

    def get_telephony_service_account_id(self):
        """Return account id for telephony service."""
        return self.config_stack.get('_twilio_account_sid')

    def get_telephony_service_auth_token(self):
        """Return auth token for telephony service."""
        return self.config_stack.get('_twilio_auth_token')

    def get_telephony_service_number1(self):
        """Return the number to call and message from."""
        return self.config_stack.get('telephony_service_number1')

    def get_telephony_service_number2(self):
        """Return the number to call and message from."""
        return self.config_stack.get('telephony_service_number2')

    def get_device_phone_number(self):
        """Return the number to send messages and calls to."""
        return self.config_stack.get('device_phone_number')
