# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging
import os

from autopilot.matchers import Eventually
from testtools import skipUnless
from testtools.matchers import Equals

import ubuntuuitoolkit
from ubuntu_system_tests.helpers import autopilot as autopilot_helpers
from ubuntu_system_tests.helpers import camera, click, images  # NOQA
from ubuntu_system_tests.helpers.backup_restore_fixture import (
    BackupRestoreFixture)
from ubuntu_system_tests.helpers.unity8.dash import get_dash
from ubuntu_system_tests.tests import base

logger = logging.getLogger(__name__)

CAMERA_CONFIG_DIR = os.path.expanduser('~/.config/com.ubuntu.camera')
CAMERA_PICTURES_DIR = os.path.expanduser('~/Pictures/com.ubuntu.camera')
CAMERA_THUMBNAILS_DIR = os.path.expanduser('~/.cache/com.ubuntu.camera')


class CameraTestCase(base.BaseUbuntuSystemTestCase):

    @skipUnless(autopilot_helpers.is_camera_supported(), 'Camera-app required')
    def setUp(self):
        """Backup existing images and settings and launch camera-app."""
        super().setUp()
        unity_proxy = self.launch_unity()
        unity_proxy.unlock()
        self.useFixture(BackupRestoreFixture(CAMERA_CONFIG_DIR))
        self.useFixture(BackupRestoreFixture(CAMERA_PICTURES_DIR))
        self.useFixture(BackupRestoreFixture(CAMERA_THUMBNAILS_DIR))
        self.camera_app = camera.MainWindow(self._launch_camera_app())

    def _launch_camera_app(self):
        """Launch the camera app from dash and return proxy object."""
        apps_scope = get_dash().open_apps_scope()
        apps_scope.click_scope_item('predefined', 'Camera')
        self.addCleanup(
            self.ensure_application_closed,
            click.get_click_app_identifier('com.ubuntu.camera'))
        return self._get_camera_app_proxy()

    def _get_camera_app_proxy(self):
        """Return camera app proxy object from existing process."""
        return autopilot_helpers.get_proxy_object_for_existing_process(
            'camera-app', ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase
        )

    def _get_last_photo_name(self):
        """Return filename for last photo taken."""
        pictures = os.listdir(CAMERA_PICTURES_DIR)
        return os.path.join(CAMERA_PICTURES_DIR, pictures[-1])

    def _check_image_not_blank(self, image):
        """Check if the taken image is blank or very limited in colours."""
        colour_count = images.get_colour_count(image)
        self.assertGreater(colour_count, 2000)

    def _check_flash_status(self, image, expected_status):
        """Check that the flash was used in the photo."""
        flash_status = images.get_flash_status_from_exif_data(image)
        self.assertThat(flash_status, Equals(expected_status))

    def _validate_photo(self, file_path):
        """Validate that the photo is not black and the flash was used."""
        self._check_image_not_blank(file_path)
        self._check_flash_status(file_path, expected_status=True)

    def test_take_picture_with_rear_camera(self):
        """
        Take a photo with the rear camera and flash enabled.
        Swipe to the photo roll gallery view and ensure photo is displayed.
        Check that the photo is not black and that the flash was used.
        """
        bottom_edge = self.camera_app.open_bottom_edge()
        self.assertThat(
            self.camera_app.get_flash_icon_name(), Equals('flash-auto'))
        self.camera_app.set_flash_option('On')
        self.assertThat(
            self.camera_app.get_flash_icon_name(), Equals('flash-on'))
        bottom_edge.close()
        photoroll_hint = self.camera_app.take_photo()
        gallery = photoroll_hint.swipe_to_dismiss()
        photo = gallery.get_photo(self._get_last_photo_name())
        self.assertThat(photo.visible, Eventually(Equals(True)))
        self.assertThat(photo.enabled, Eventually(Equals(True)))
        self.assertFalse(photo.isVideo)
        gallery.go_back()
        self.camera_app.ensure_camera_ready()
        self._validate_photo(self._get_last_photo_name())
