
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from autopilot.matchers import Eventually
from testtools.matchers import Equals, NotEquals
from testtools import skipUnless

from ubuntu_system_tests.helpers import (
    autopilot,
    indicators,
    ofono,
    radio_killswitch,
)
from ubuntu_system_tests.tests import base


class FlightModeTestCase(base.BaseUbuntuSystemTestCase):

    @skipUnless(ofono.is_sim_inserted() and autopilot.is_wifi_supported(),
                'SIM and Wi-Fi support required.')
    def setUp(self):
        super().setUp()
        self.radio_killswitch = self.useFixture(
            radio_killswitch.RadioKillswitch(False))

    def assert_network_indicator_shows_flightmode_off(self,
                                                      network_indicator_page):
        """Network indicator UI elements reflect 'off' state of flight mode."""
        self.assertTrue(network_indicator_page.is_flight_mode_switch_visible())
        self.assertThat(network_indicator_page.is_sim_list_offline,
                        Eventually(Equals(False)))
        self.assertTrue(network_indicator_page.is_wifi_switch_visible())
        self.assertThat(network_indicator_page.is_wifi_enabled,
                        Eventually(Equals(True)))
        self.assertThat(network_indicator_page.get_list_of_wifi_access_points,
                        Eventually(NotEquals([])))
        self.assertTrue(network_indicator_page.is_cellular_settings_visible())
        self.assertTrue(network_indicator_page.is_wifi_settings_visible())

    def assert_network_indicator_shows_flightmode_on(self,
                                                     network_indicator_page):
        """Network indicator UI elements reflect 'on' state of flight mode."""
        self.assertThat(network_indicator_page.is_sim_list_offline,
                        Eventually(Equals(True)))
        self.assertThat(network_indicator_page.is_wifi_enabled,
                        Eventually(Equals(False)))
        self.assertThat(
            network_indicator_page.get_list_of_wifi_access_points,
            Eventually(Equals([])))

    def test_switching_on_flight_mode_must_engage_flight_mode(self):
        """Switching on 'flight mode' in the indicator must engage flight mode.

        Note that we assume we're starting in *off* mode; also we skip
        the test if no SIM is inserted.

        """
        unity = self.launch_unity()
        self.assertFalse(self.radio_killswitch.is_flight_mode_on(),
                         'Test must be run with flight mode off.')
        network_indicator = indicators.NetworkIndicator(unity.main_window)
        network_indicator_page = network_indicator.open()
        self.addCleanup(network_indicator.ensure_closed)
        self.assert_network_indicator_shows_flightmode_off(
            network_indicator_page)
        network_indicator_page.turn_on_flight_mode()
        self.assertThat(self.radio_killswitch.is_flight_mode_on,
                        Eventually(Equals(True)))
        self.assert_network_indicator_shows_flightmode_on(
            network_indicator_page)
        network_indicator_page.turn_off_flight_mode()
        self.assertThat(self.radio_killswitch.is_flight_mode_on,
                        Eventually(Equals(False)))
        self.assert_network_indicator_shows_flightmode_off(
            network_indicator_page)
