# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import uuid
from datetime import datetime

from autopilot.exceptions import StateNotFoundError
from autopilot.matchers import Eventually
from messaging_app import emulators as messaging_emulators
from messaging_app import fixture_setup as messaging_fixtures
from testtools import skipUnless
from testtools.matchers import Equals
from ubuntuuitoolkit import emulators as toolkit_emulators

from ubuntu_system_tests.helpers.addressbook import dbus_service
from ubuntu_system_tests.helpers import autopilot as autopilot_helpers
from ubuntu_system_tests.helpers.messaging import fixture_setup
from ubuntu_system_tests.helpers.messaging import messaging as messaging_helpers  # NOQA
from ubuntu_system_tests.helpers import file_system
from ubuntu_system_tests.helpers import ofono
from ubuntu_system_tests.helpers import web
from ubuntu_system_tests.helpers.telephony import Telephony
from ubuntu_system_tests.helpers.unity8 import dash, notifications  # NOQA

from ubuntu_system_tests.tests import base

CONTACT_NAME_1 = 'Fulano de Tal'
CONTACT_NAME_2 = 'Ubuntu Tester'
TIMEOUT_5_MIN = 300


class MessagingTestCase(base.BaseTelephonyTestCase):

    @skipUnless(ofono.is_sim_inserted(), 'No SIM inserted')
    def setUp(self):
        super().setUp()
        self.unity_proxy = self.restart_unity()
        self.useFixture(fixture_setup.MessagingTestEnvironment())
        credentials = {'AccountSid': self.get_telephony_service_account_id(),
                       'AuthToken': self.get_telephony_service_auth_token()}
        self.telephony = Telephony(credentials)
        self._create_temporary_contact(
            CONTACT_NAME_1,
            self.get_telephony_service_number1())
        # This disables the gestures helper which
        # appears when first sending a message
        self.useFixture(messaging_fixtures.UseDefaultConfiguration())

    def _launch_messaging(self):
        dash_proxy = dash.get_dash()
        apps_scope = dash_proxy.open_apps_scope()
        apps_scope.click_scope_item('predefined', 'Messaging')
        return self._get_messaging_app_main_view_and_cleanup()

    def _get_messaging_app_main_view_and_cleanup(self):
        """Return messaging app proxy object from existing process."""
        proxy = autopilot_helpers.get_proxy_object_for_existing_process(
            'messaging-app',
            toolkit_emulators.UbuntuUIToolkitEmulatorBase)
        return proxy.wait_select_single(messaging_emulators.MainView)

    def _create_temporary_contact(self, contact_name, phone_number):
        """
        Create a contact in the address book using the dbus service
        and remove it in the cleanup step.
        """
        vcard = """
BEGIN:VCARD
VERSION:3.0
FN:{name}
TEL;TYPE=CELL:{number}
END:VCARD""".format(name=contact_name,
                    number=phone_number)
        self.addressbook_dbus = dbus_service.ContactsDbusService()
        contact_vcard = self.addressbook_dbus.create_contact(vcard)
        contact_uid = self.addressbook_dbus.get_vcard_uid(contact_vcard)
        self.addCleanup(self.addressbook_dbus.delete_contact, [contact_uid])

    def _get_notifications_list(self):
        """Return a list of notifications currently being displayed."""
        return self.unity_proxy.select_single(
            'Notifications',
            objectName='notificationList'
        )

    def _receive_message_and_get_notification(self, to, from_, message,
                                              media_url=None, timeout=60):
        """
        Receive a message from Telephony service and return notification.

        If media_url is None then an SMS will be sent, if media_url is
        specified the content will be attached to an MMS message.

        :param to: Phone number of recipient
        :param from_: Phone number of sender
        :param message: Body of the message to send
        :param media_url: Url of file to attach to message
        :param timeout: Timeout period to wait for notification in seconds
        :return: Notification message
        """
        notifications_list = self._get_notifications_list()
        if media_url:
            sent = self.telephony.send_mms(to, from_, message, media_url)
        else:
            sent = self.telephony.send_sms(to, from_, message)

        self.assertTrue(sent, 'The message was not sent succesfully by the '
                              'telephony service.')

        return notifications_list.get_current_notification(timeout)

    def test_receive_sms_from_contact(self):
        """
        Prerequisite: A contact in the address book with the number you
        are sending from
        1. From another device send an SMS to the phone number of a SIM
           card that is in the SUT
           - A notification appears containing the senders
             name and the message content.
        2. Tap on the message notification before it disappears
           - The messaging application is opened showing all
             messages received by that sender. The message
             just sent should appear at the bottom.
        """
        message_body = str(uuid.uuid1())
        notification = self._receive_message_and_get_notification(
            self.get_device_phone_number(),
            self.get_telephony_service_number1(),
            message_body,
            timeout=TIMEOUT_5_MIN)

        self.assertThat(notification.summary,
                        Eventually(Equals('Message from {contact}'.format(
                                          contact=CONTACT_NAME_1))))
        self.assertThat(notification.body, Eventually(Equals(message_body)))

        # Open the messaging app by tapping on the notification.
        # Due to launchpad.net/bugs/1457629 the following tap_on() method can
        # fail due to CPO inheritance problems in Autopilot.
        # A workaround is to select and press the object directly in the test.
        # This should be removed as soon as the Autopilot CPO bug is fixed.
        # notification.tap_on()
        notification.pointing_device.click_object(
            notification.select_single(objectName="interactiveArea"))

        messaging_app = self._get_messaging_app_main_view_and_cleanup()
        messaging_app.get_message(message_body)
        self.assertThat(messaging_app.get_header().title,
                        Eventually(Equals(CONTACT_NAME_1)))

    def _clear_provider_notifications(self):
        while True:
            notification_list = self._get_notifications_list()
            try:
                provider_notif = notification_list.get_current_notification(
                    timeout=10
                )
            except StateNotFoundError:
                return
            provider_notif.press_ok()

    def is_message_received(self, text):
        """
        Check if the message with the specified text
        has been received yet

        :param text: The body of the message we are expecting
        :return True if received, otherwise False
        """
        messages = self.main_view.get_messages_page().get_messages()
        if messages[0][1] == text:
            return True
        return False

    def test_send_sms_to_contact(self):
        """
        Prerequisite: A contact in the address book with the number you
                     are sending to
        1. Open the messaging application
            - The messaging application is opened to the main screen
        2. Swipe up to create a new message
            - The keyboard appears with the message composition controls
              above it (add photo button, 'Write a message...' field)
              and a Send button. The To.. field has focus.
        3. Type the name of the contact to send to in the To.. field
            - The contacts name is stored in the To... field
        4. Type the message text in the Write a message..
           field - e.g. 'Hello World'
            - The message text is entered into the Write a message.. field
        5. Press the Send button
            - The message is added to the conversation and is shown in green
        6. Confirm the message was received on the other device
            - The message was received with the correct text
        """
        # Open the messaging app
        self.main_view = self._launch_messaging()
        # Potentially select the SIM to use if prompted
        self.main_view.select_sim_if_prompted(1)
        # Send the message to the contact
        message = str(uuid.uuid1())
        self.main_view.start_new_message()
        self.main_view.type_contact_name_and_select_num(
            CONTACT_NAME_1,
            self.get_telephony_service_number1())
        self.main_view.type_message(message)
        self.main_view.click_send_button()
        # There may be a notification we need to clear
        # depending on the provider used
        self._clear_provider_notifications()
        # Wait for response to be received
        received = self.wait_until(
            self.is_message_received, timeout=60, period=5, text='ACK'
        )
        self.assertTrue(received,
                        'Did not receive response from '
                        'the telephony service.')
        # Query Twilio for the message we sent
        sent_sms = self.telephony.get_sent_sms(
            self.get_telephony_service_number1(),
            self.get_device_phone_number(),
            datetime.today(),
            message)
        self.assertTrue(sent_sms)

    def _is_sms_received(self, text, number):
        thread_list = self.main_view.get_thread_list()
        self.assertThat(thread_list.visible, Eventually(Equals(True)))

        thread_labels = self.main_view.get_all_labels_for_thread(number)
        if not thread_labels:
            self.fail("Thread not found for number: {}".format(number))
        for label in thread_labels:
            if label.text == text:
                return True
        return False

    def test_send_sms_to_multiple_contacts(self):
        """
        Prerequisite: Two contacts in the address book with the numbers you
                     are sending to
        1. Open the messaging application
            - The messaging application is opened to the main screen
        2. Swipe up to create a new message
            - The keyboard appears with the message composition controls
              above it (add photo button, 'Write a message...' field)
              and a Send button. The To.. field has focus.
        3. Type the name of the first contact to send to in the To.. field
            - The contacts name is stored in the To... field
        4. Type the name of the second contact to send to in the To.. field
            - The contacts name is stored in the To... field
        5. Type the message text in the Write a message..
           field - e.g. 'Hello World'
            - The message text is entered into the Write a message.. field
        6. Press the Send button
            - The message is added to the conversation and is shown in green
        7. Confirm the message was received on the other device
            - The message was received with the correct text
        """
        number_1 = self.get_telephony_service_number1()
        number_2 = self.get_telephony_service_number2()
        self._create_temporary_contact(CONTACT_NAME_2, number_2)
        # Open the messaging app
        self.main_view = self._launch_messaging()
        # Potentially select the SIM to use if prompted
        self.main_view.select_sim_if_prompted(1)
        # Send the message to the contact
        message = str(uuid.uuid1())
        self.main_view.start_new_message()
        self.main_view.type_contact_name_and_select_num(
            CONTACT_NAME_1,
            number_1)
        self.main_view.type_contact_name_and_select_num(
            CONTACT_NAME_2,
            number_2)
        self.main_view.type_message(message)
        self.main_view.click_send_button()
        # There may be a notification we need to clear
        # depending on the provider used
        self._clear_provider_notifications()
        # Delete the newly sent message so that its thread
        # is not shown.
        self.main_view.delete_message(message)
        self.main_view.go_back()
        # Wait for response to be received
        received_sms_1 = self.wait_until(
            self._is_sms_received,
            timeout=60,
            period=5,
            text='ACK',
            number=number_1
        )
        self.assertTrue(received_sms_1,
                        'Did not receive response from '
                        'the telephony service.')
        received_sms_2 = self.wait_until(
            self._is_sms_received,
            timeout=60,
            period=5,
            text='ACK2',
            number=number_2
        )
        self.assertTrue(received_sms_2,
                        'Did not receive response from '
                        'the telephony service.')
        # Query Twilio for the message we sent
        self.assertTrue(self.telephony.get_sent_sms(
            number_1,
            self.get_device_phone_number(),
            datetime.today(),
            message))
        self.assertTrue(self.telephony.get_sent_sms(
            number_2,
            self.get_device_phone_number(),
            datetime.today(),
            message))

    def test_receive_mms_from_contact(self):
        """
        Prerequisite: A contact in the address book with the number you
        are sending from
        1. From another device send an MMS to the phone number of a SIM
           card that is in the SUT
           - A notification appears containing the senders
             name and the message content.
        2. Tap on the message notification before it disappears
           - The messaging application is opened showing all
             messages received by that sender. The message
             just sent should appear at the bottom.
           - An image should be contained within the message
        """
        message_body = str(uuid.uuid1())
        media_url = 'http://people.canonical.com/~platform-qa/mms-ubuntu.jpg'
        mms_media_file = web.download_file(media_url)
        notification = self._receive_message_and_get_notification(
            self.get_device_phone_number(),
            self.get_telephony_service_number1(),
            message_body,
            media_url=media_url,
            timeout=TIMEOUT_5_MIN)

        self.assertThat(notification.summary,
                        Eventually(Equals('Message from {contact}'.format(
                                          contact=CONTACT_NAME_1))))
        self.assertThat(notification.body, Eventually(Equals(message_body)))

        # Open the messaging app by tapping on the notification.
        # Due to launchpad.net/bugs/1457629 the following tap_on() method can
        # fail due to CPO inheritance problems in Autopilot.
        # A workaround is to select and press the object directly in the test.
        # This should be removed as soon as the Autopilot CPO bug is fixed.
        # notification.tap_on()
        notification.pointing_device.click_object(
            notification.select_single(objectName="interactiveArea"))

        messaging_app = self._get_messaging_app_main_view_and_cleanup()
        self.assertThat(messaging_app.get_header().title,
                        Eventually(Equals(CONTACT_NAME_1)))

        mms_message = messaging_app.get_message(message_body)
        photo_path = mms_message.get_mms_content_file_path()
        self.assertTrue(file_system.compare_files(mms_media_file, photo_path))
