# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from collections import namedtuple
import logging
import os

from ubuntu_system_tests import config
from ubuntu_system_tests.helpers import file_system as fs
from ubuntu_system_tests.helpers import ssh
from ubuntu_system_tests.helpers.scopes import music
from ubuntu_system_tests.helpers.unity8.dash import get_dash

from ubuntu_system_tests.tests import base

TEMP_DIR = '/tmp'
DEFAULT_XDG_RUNTIME_DIR = '/run/user/1000'

MUSIC = "Music"
MUSIC_DIR = os.path.join(os.path.expanduser("~"), MUSIC)
DOCS = "Documents"
DOCS_DIR = os.path.join(os.path.expanduser("~"), DOCS)
PICTURES = "Pictures"
PICTURES_DIR = os.path.join(os.path.expanduser("~"), PICTURES)
VIDEOS = "Videos"
VIDEOS_DIR = os.path.join(os.path.expanduser("~"), VIDEOS)

FileMusicDesc = namedtuple('FileMusicDesc', ['file', 'album'])
logger = logging.getLogger(__name__)
songs_album_mp3 = ['song_1.mp3', 'song_2.mp3', 'song_3.mp3']
songs_album_ogg = ['song_1.ogg', 'song_2.ogg', 'song_3.ogg']
test_data = {
    'mp3_file': FileMusicDesc(
        file='hello.mp3',
        album='Canonical'),
    'ogg_file': FileMusicDesc(
        file='hello.ogg',
        album='Ubuntu'),
    'text_file': 'hello.txt',
}


class MTPFile:
    """ This class is a data model which represents a file to be copied
    through mtp
    """
    def __init__(self, remote_helper):
        self.remote_helper = remote_helper
        self.local_path = ""
        self.remote_src_path = ""
        self.remote_dst_path = ""
        self.sha1 = ""

    @property
    def name(self):
        return os.path.basename(self.local_path) or ""

    def clean_file(self):
        if self.local_path:
            fs.delete_file(self.local_path)
        if self.remote_src_path:
            self.remote_helper.remove_file(self.remote_src_path)
        if self.remote_dst_path:
            self.remote_helper.remove_file(self.remote_dst_path)

    def remote_copy(self):
        if not self.remote_src_path or not self.remote_dst_path:
            raise RuntimeError("Remote copy cannot be done, it is required to "
                               "define src and dst paths,")
        logger.info("Copying remote file from {src_path} to {dst_path}".
                    format(src_path=self.remote_src_path,
                           dst_path=self.remote_dst_path))
        self.remote_helper.copy_file(self.remote_src_path,
                                     self.remote_dst_path)

    def get_remote_dst_sha1(self):
        if not self.remote_dst_path:
            raise RuntimeError("Attribute remote_dst_path has to de defined")
        return self.remote_helper.get_sha1(self.remote_dst_path)


class MTPTest(base.BaseUbuntuSystemTestCase):
    """This Class tests the mtp protocol"""

    def setUp(self):
        super().setUp()

        self.config_stack = config.get_device_config_stack()
        self.unity_proxy = self.restart_unity()
        self.remote_helper = ssh.LinuxRemoteHelper(
            self.config_stack.get('ssh_ip'),
            self.config_stack.get('ssh_user'),
            self.config_stack.get('ssh_password'))
        self.addCleanup(self.remote_helper.close)
        self.mtp_dir = self.calculate_device_mtp_dir()

    def create_random_file(self, dir, ext='', sha1=True):
        file = MTPFile(self.remote_helper)
        file.local_path = fs.create_random_file(dir, ext)
        if sha1:
            file.sha1 = fs.calculate_file_sha1(file.local_path)
        self.addCleanup(file.clean_file)
        return file

    def get_first_elem_in_dir_command(self, path):
        """Command to retrieve the first element in a dir"""
        return 'ls {} | sort -n | head -1'.format(path)

    def get_device_mtp_dir(self, dir_to_map, filename):
        """ Determine the full_path in the ubuntu host machine which maps the
        file and directory in the device
        :param dir_to_map: The directory to map
        :param filename: The name of the file to be mapped
        :return: the full path that maps the dir_to_map and filename in the
                 device
        """
        cmd_line = "python3 -c \"import glob; import os; " \
                   "print(glob.glob(os.getenv('XDG_RUNTIME_DIR', " \
                   "'{default}') + '/gvfs/*/*/{dir}/{filename}')[0])\"".\
            format(default=DEFAULT_XDG_RUNTIME_DIR,
                   dir=dir_to_map,
                   filename=filename)

        file_path = self.remote_helper.run(cmd_line).strip()
        if file_path:
            return os.path.dirname(file_path)
        return None

    def calculate_device_mtp_dir(self):
        """ Get the path in the ubuntu host machine which maps to the mtp dir.
        Use a random file to identify the device internal storage destination.
        """
        file = fs.create_random_file(DOCS_DIR, ext='.tmp')
        self.addCleanup(os.remove, file)
        docs_mtp_dir = self.get_device_mtp_dir(DOCS, os.path.basename(file))
        return os.path.dirname(docs_mtp_dir)

    def get_device_mtp_path(self, *paths):
        """ Retrieve the mtp path following the desired paths
        :param paths: A list of subdir to the destination path
        :return: The mtp path already escaped to the destination path
        """
        dir = os.path.join(self.mtp_dir, *paths)
        return '"' + dir + '"'

    def verify_music_album_is_shown(self, album):
        """ Determines if there is an album with the desired name in the music
        scope
        :param album: The name ob the desired album
        :return: True is there is an album with the desired name
        """
        music_scope = music.MusicScope(get_dash().open_music_scope())
        self.assertTrue(music_scope.is_album(album))

    def create_remote_dir(self, parent_dir):
        """
        Create a dir with random name in the remote host
        :param parent_dir: the name of the parent dir where the new dir has to
        be created
        :return: the dir name and its full path
        """
        dir_name = fs.get_random_string()
        dir_path = os.path.join(parent_dir, dir_name)
        self.remote_helper.create_dir(dir_path)

        return dir_name, dir_path

    def get_audio_dir(self):
        """ Get the test data audio dir
        :return: The dir where all the audio test data is stored
        """
        return os.path.join(self.get_tests_dir(), 'data', 'audio')

    def get_text_test_data_dir(self):
        """ Get the test dir for text data"""
        return os.path.join(self.get_tests_dir(), 'data', 'text')

    def copy_file_to_remote_host(self, file_path, remote_dir):
        """ Copy a local file to a remote host
        :param file_path: the local dire where the file is located
        :param remote_dir: the remote dir where the file is going to be copied
        :return: the full path of the file in the remote host
        """
        dst_file = os.path.join(remote_dir, os.path.basename(file_path))
        logger.info("Copying local file {file_path} to remote host on "
                    "{dst_file}".format(file_path=file_path,
                                        dst_file=dst_file))
        self.remote_helper.copy(file_path, dst_file)
        self.addCleanup(self.remote_helper.remove_file, dst_file)
        return dst_file

    def verify_same_files(self, src_files, dst_files):
        """ Verify both file lists contains the same files
        :param src_files: list of src file names
        :param dst_files: list of dst file names
        """
        self.assertEqual(sorted(src_files), sorted(dst_files))

    def verify_files_match(self, src_file, dst_dir, dst_files):
        """ Verify the src file is equal to the dst files in the dst_dir
        :param src_file: file from where the original files were copied
        :param dst_dir: name of the dir copied to the device
        :param dst_files: list of files copied to the copied in the device
        """
        for file in dst_files:
            self.assertTrue(fs.compare_files(src_file,
                                             os.path.join(dst_dir, file)))

    def replicate_file(self, src_file, dst_dir, dst_files):
        """ Copy the src file to the dst directory with the different names
        stored in the dst_files variable
        :param src_file: the source file to replicate
        :param dst_dir: the destination directory to copy the files
        :param dst_files: the target names to use
        """
        for file in dst_files:
            self.remote_helper.copy_file(src_file,
                                         os.path.join(dst_dir, file))

    def verify_file_copied(self, base_dir, file_name):
        """ Verify the file was copied to the base directory
        :param base_dir: the directory where the file has to be copied
        :param file_name: the name of the copied file
        :return: the destination path where the file was copied
        """
        copied_file = os.path.join(base_dir, file_name)
        self.assertTrue(os.path.isfile(copied_file))
        self.addCleanup(os.remove, copied_file)
        return copied_file

    def verify_dir_copied(self, base_dir, dir_name):
        """ Verify the dir was copied to the base directory
        :param base_dir: the directory where the dir name has to be copied
        :param dir_name: the name of the copied dir
        :return: the destination path where the dir was copied
        """
        copied_dir = os.path.join(base_dir, dir_name)
        self.assertTrue(os.path.isdir(copied_dir))
        self.addCleanup(fs.remove_dir, copied_dir)
        return copied_dir

    def prepare_music_albums(self, src_mp3_file, src_ogg_file):
        """ Prepare the albums in the remote host with the song to be copied
         It creates one dir with two albums inside, each album has 3 songs with
         mp3 and ogg formats respectively
        :return: the
        """

        # Create music albums structure in the host machine
        (albums_name, albums_dir) = self.create_remote_dir(TEMP_DIR)
        (album_mp3_name, album_mp3_dir) = self.create_remote_dir(albums_dir)
        (album_ogg_name, album_ogg_dir) = self.create_remote_dir(albums_dir)
        self.addCleanup(self.remote_helper.remove_dir, albums_dir)

        # Copy the music files to the ubuntu host machine
        dst_tmp_mp3_file = \
            self.copy_file_to_remote_host(src_mp3_file, TEMP_DIR)
        dst_tmp_ogg_file = \
            self.copy_file_to_remote_host(src_ogg_file, TEMP_DIR)

        # Copy the music files in the ubuntu host machine to the albums
        # several times with different names and formats
        self.replicate_file(dst_tmp_mp3_file, album_mp3_dir, songs_album_mp3)
        self.replicate_file(dst_tmp_ogg_file, album_ogg_dir, songs_album_ogg)

        return albums_name, albums_dir, album_mp3_name, album_ogg_name

    def verify_copied_files(self, seed_file, original_files, copied_dir):
        """ Verify all the files have been copied correctly to the dst
        :param seed_file: this is the test data file that is used to generate
        the test data for the test. This is used to verify the integrity of
        the files after those are copied through mtp.
        :param original_files: list of files that were not copied
        :param copied_dir: mtp dir to where the original files were copied
        """
        copied_files = fs.files_in_dir(copied_dir)

        self.verify_same_files(original_files, copied_files)
        self.verify_files_match(seed_file, copied_dir, copied_files)

    def verify_songs(self, seed_song, original_songs, copied_album, album):
        """ Verify all the songs have been copied for each album correctly and
        verify the albums are in the music scope
        """
        self.verify_copied_files(seed_song, original_songs, copied_album)
        self.verify_music_album_is_shown(album)

    def test_copy_music(self):
        """ Copy through mtp a file to the music dir and check the file has
        been copied to the correct dir and the content is the same to the
        original one

        This test initially creates through ssh two temporal music albums in
        the remote host machine with mp3 and ogg songs respectively. Then,
        these music albums are copied to the gvfs dir which maps the device
        music directory. Once those are copied, it is checked that the music
        albums have been copied properly to the device and the music appears
        in the music scope.
        """

        # Prepare the albums structure in the remote machine previous to be
        # copied to the device
        src_mp3_file = os.path.join(self.get_audio_dir(),
                                    test_data.get('mp3_file').file)
        src_ogg_file = os.path.join(self.get_audio_dir(),
                                    test_data.get('ogg_file').file)
        albums_name, albums_dir, album_mp3_name, album_ogg_name = \
            self.prepare_music_albums(src_mp3_file, src_ogg_file)

        # Copy the albums directory from the temporal dir in the ubuntu host
        # machine to the dir which maps the music dir in the device
        device_mtp_albums_path = self.get_device_mtp_path(MUSIC, albums_name)
        self.remote_helper.copy_dir(albums_dir, device_mtp_albums_path)

        # Verify the albums have ben copied to the music folder
        copied_albums = self.verify_dir_copied(MUSIC_DIR, albums_name)

        copied_album_mp3 = os.path.join(copied_albums, album_mp3_name)
        self.verify_songs(src_mp3_file, songs_album_mp3, copied_album_mp3,
                          test_data.get('mp3_file').album)

        copied_album_ogg = os.path.join(copied_albums, album_ogg_name)
        self.verify_songs(src_ogg_file, songs_album_ogg, copied_album_ogg,
                          test_data.get('ogg_file').album)

    def test_copy_large_number_of_files(self):
        """ Copy big amount of files through mtp and check all the files have
        been copied to the correct dir and the content is the same to the
        original.
        """
        amount = 100
        src_file_name = test_data.get('text_file')
        src_file = os.path.join(self.get_text_test_data_dir(), src_file_name)

        # Create the dir to store the files in the host machine
        (dir_name, dir_path) = self.create_remote_dir(TEMP_DIR)
        self.addCleanup(self.remote_helper.remove_dir, dir_path)

        # Copy the src file to the ubuntu host machine
        dst_tmp_file = self.copy_file_to_remote_host(src_file, TEMP_DIR)

        # Replicate the src file 100 times
        original_files = [src_file_name + "_" + str(i) for i in range(amount)]
        self.replicate_file(dst_tmp_file, dir_path, original_files)

        # Copy the directory from the temporal dir in the ubuntu host
        # machine to the dir which maps the documents dir in the device
        device_mtp_dir_path = self.get_device_mtp_path(DOCS, dir_name)
        logger.info("Copying remote files to mtp directory from {src_path} to "
                    "{mtp_path}".format(src_path=dir_path,
                                        mtp_path=device_mtp_dir_path))
        self.remote_helper.copy_dir(dir_path, device_mtp_dir_path)

        # Verify the files have ben copied correctly to the destiation
        copied_dir = self.verify_dir_copied(DOCS_DIR, dir_name)
        self.verify_copied_files(src_file, original_files, copied_dir)

    def _test_copy_big_file(self, kb=0, mb=0, gb=0):
        # Create the remote file with random content and get the sha1 digest
        file_name = fs.get_random_string() + ".tmp"
        file_path = os.path.join(TEMP_DIR, file_name)

        self.remote_helper.create_random_file(file_path, kb=kb, mb=mb, gb=gb)
        self.addCleanup(self.remote_helper.remove_file, file_path)
        file_sha1 = self.remote_helper.get_sha1(file_path).strip()

        # Copy the directory from the temporal dir in the ubuntu host
        # machine to the dir which maps the documents dir in the device
        device_mtp_dst_path = self.get_device_mtp_path(DOCS, file_name)
        logger.info("Copying remote files to mtp directory from {src_path} to "
                    "{mtp_path}".format(src_path=file_path,
                                        mtp_path=device_mtp_dst_path))
        self.remote_helper.copy_file(file_path, device_mtp_dst_path)

        # Verify the files have ben copied correctly to the destiation
        copied_file = self.verify_file_copied(DOCS_DIR, file_name)
        copied_sha1 = fs.calculate_file_sha1(copied_file)
        self.assertEqual(file_sha1, copied_sha1)

    def test_copy_big_1gb_file(self):
        """ Copy a big file (almost 2GB) through mtp and check the file has
        been copied to the correct dir and the content is the same to the
        original.
        """
        self._test_copy_big_file(gb=1)

    def test_copy_big_2gb_file(self):
        """ Copy a big file (2GB) through mtp and check the file has been
        copied to the correct dir and the content is the same to the original.

        This test is failing and a bug has been created
        https://bugs.launchpad.net/ubuntu/+source/mtp/+bug/1472789
        """
        self._test_copy_big_file(gb=2)

    def test_copy_big_4gb_file(self):
        """ Copy a big file (4GB) through mtp and check the file has been
        copied to the correct dir and the content is the same to the original.

        This test is failing and a bug has been created
        https://bugs.launchpad.net/ubuntu/+source/mtp/+bug/1472789
        """
        self._test_copy_big_file(gb=4)

    def test_copy_from_device(self):
        """ Copy Music, Videos and Pictures from the device to the ubuntu host
         machine and delete the files in the device from the host
        """

        # Create random source data and calculate the sha1 digest
        song = self.create_random_file(MUSIC_DIR, ext='.mp3')
        song.remote_src_path = self.get_device_mtp_path(MUSIC, song.name)

        video = self.create_random_file(VIDEOS_DIR, ext='.avi')
        video.remote_src_path = self.get_device_mtp_path(VIDEOS, video.name)

        picture = self.create_random_file(PICTURES_DIR, ext='.jpg')
        picture.remote_src_path = self.get_device_mtp_path(PICTURES,
                                                           picture.name)

        files = [song, video, picture]

        # Set destination path to files
        for file in files:
            file.remote_dst_path = os.path.join(TEMP_DIR, file.name)

        # Copy the files from mtp to tmp directory and verify the sha1
        for file in files:
            file.remote_copy()
            copied_sha1 = file.get_remote_dst_sha1()
            self.assertEqual(file.sha1, copied_sha1)

        # Remove remote files and verify the files were deleted in the device
        for file in files:
            self.remote_helper.remove_file(file.remote_src_path)
            self.assertFalse(os.path.isfile(file.local_path))
